generate_md_episodes <- function() {

    library("methods")
    
    if (require("knitr") && packageVersion("knitr") < '1.9.19')
        stop("knitr must be version 1.9.20 or higher")

    if (!require("stringr"))
        stop("The package stringr is required for generating the lessons.")

    if (require("checkpoint") && packageVersion("checkpoint") >=  '0.4.0') {
        required_pkgs <-
             checkpoint:::scanForPackages(project = "_episodes_rmd",
                                          verbose=FALSE, use.knitr = TRUE)$pkgs
    } else {
        stop("The checkpoint package (>= 0.4.0) is required to build the lessons.")
    }

    missing_pkgs <- required_pkgs[!(required_pkgs %in% rownames(installed.packages()))]

    if (length(missing_pkgs)) {
        message("Installing missing required packages: ",
                paste(missing_pkgs, collapse=", "))
        install.packages(missing_pkgs)
    }

    ## get the Rmd file to process from the command line, and generate the path for their respective outputs
    args  <- commandArgs(trailingOnly = TRUE)
    if (!identical(length(args), 2L)) {
      stop("input and output file must be passed to the script")
    }
    
    src_rmd <- args[1]
    dest_md <- args[2]

    ## knit the Rmd into markdown
    knitr::knit(src_rmd, output = dest_md)

    # Read the generated md files and add comments advising not to edit them
    vapply(dest_md, function(y) {
      con <- file(y)
      mdfile <- readLines(con)
      if (mdfile[1] != "---")
        stop("Input file does not have a valid header")
      mdfile <- append(mdfile, "# Please do not edit this file directly; it is auto generated.", after = 1)
      mdfile <- append(mdfile, paste("# Instead, please edit", 
                                     basename(y), "in _episodes_rmd/"), after = 2)
      writeLines(mdfile, con)
      close(con)
      return(paste("Warning added to YAML header of", y))
    },
    character(1))
}

generate_md_episodes()
